import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { FooterCmsRepository } from './footer-cms.repository';
import { UpdateFooterCmsDto } from './dto/update-footer-cms.dto';
import { FooterCms } from './entities/footer-cms.entity';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class FooterCmsService {
    constructor(
        @InjectRepository(FooterCmsRepository)
        private readonly repository: FooterCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async updateOrCreateData(
        dto: UpdateFooterCmsDto,
        files: {
            logo: UploadedMulterFileI,
            paymentMethodImage: UploadedMulterFileI,
        }
    ): Promise<FooterCms | Error> {
        // process image one
        if (files && files.logo) {
            const logo: any = await this.doSpaceService.uploadFile(files.logo[0], 'FooterCms/');
            dto.logo = logo;
        }

        // process image two
        if (files && files.paymentMethodImage) {
            const paymentMethodImage: any = await this.doSpaceService.uploadFile(files.paymentMethodImage[0], 'FooterCms/');
            dto.paymentMethodImage = paymentMethodImage;
        }

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<FooterCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Footer cms data fetched successfully", "data", data);
    }
}
