import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { HomePageSectionThreeCmsRepository } from './home-page-section-three-cms.repository';
import { UpdateHomePageSectionThreeCmsDto } from './dto/update-home-page-section-three-cms.dto';
import { HomePageSectionThreeCms } from './entities/home-page-section-three-cms.entity';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class HomePageSectionThreeCmsService {
    constructor(
        @InjectRepository(HomePageSectionThreeCmsRepository)
        private readonly repository: HomePageSectionThreeCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async updateOrCreateData(
        dto: UpdateHomePageSectionThreeCmsDto,
        files: {
            imageOne: UploadedMulterFileI,
            imageTwo: UploadedMulterFileI,
        }
    ): Promise<HomePageSectionThreeCms | Error> {
        // process image one
        if (files && files.imageOne) {
            const imageOne: any = await this.doSpaceService.uploadFile(files.imageOne[0], 'HomePageSectionThreeCms/');
            dto.imageOne = imageOne;
        }

        // process image two
        if (files && files.imageTwo) {
            const imageTwo: any = await this.doSpaceService.uploadFile(files.imageTwo[0], 'HomePageSectionThreeCms/');
            dto.imageTwo = imageTwo;
        }

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<HomePageSectionThreeCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Home page section Three cms data fetched successfully", "data", data);
    }
}
