import { Controller, Get, Post, Body, HttpCode, UseGuards, HttpStatus, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { ShopGuideCmsService } from './shop-guide-cms.service';
import { UpdateShopGuideCmsDto } from './dto/update-shop-guide-cms.dto';

@Controller({ path: 'shop-guide-cms', version: Constants.API_VERSION_1 })

export class ShopGuideCmsController {
    constructor(private readonly service: ShopGuideCmsService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'image', maxCount: 1 }
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdateShopGuideCmsDto,
        @UploadedFiles() files: { 
            image: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
