import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { StylePageSectionOneCmsRepository } from './style-page-section-one-cms.repository';
import { UpdateStylePageSectionOneCmsDto } from './dto/update-style-page-section-one-cms.dto';
import { StylePageSectionOneCms } from './entities/style-page-section-one-cms.entity';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Injectable()
export class StylePageSectionOneCmsService {
    constructor(
        @InjectRepository(StylePageSectionOneCmsRepository)
        private readonly repository: StylePageSectionOneCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService
    ) { }

    async updateOrCreateData(
        dto: UpdateStylePageSectionOneCmsDto,
        files: { 
            imageOne: UploadedMulterFileI, 
            imageTwo: UploadedMulterFileI, 
            imageThree: UploadedMulterFileI, 
            imageFour: UploadedMulterFileI, 
            imageFive: UploadedMulterFileI
        }
    ): Promise<StylePageSectionOneCms | Error> {
        // process image one
        if (files && files.imageOne) {
            const imageOne: any = await this.doSpaceService.uploadFile(files.imageOne[0], 'stylePageSectionOneCms/');
            dto.imageOne = imageOne;
        }
        // process image two
        if (files && files.imageTwo) {
            const imageTwo: any = await this.doSpaceService.uploadFile(files.imageTwo[0], 'stylePageSectionOneCms/');
            dto.imageTwo = imageTwo;
        }

        // process image three
        if (files && files.imageThree) {
            const imageThree: any = await this.doSpaceService.uploadFile(files.imageThree[0], 'stylePageSectionOneCms/');
            dto.imageThree = imageThree;
        }

        // process image four
        if (files && files.imageFour) {
            const imageFour: any = await this.doSpaceService.uploadFile(files.imageFour[0], 'stylePageSectionOneCms/');
            dto.imageFour = imageFour;
        }

        // process image five
        if (files && files.imageFive) {
            const imageFive: any = await this.doSpaceService.uploadFile(files.imageFive[0], 'stylePageSectionOneCms/');
            dto.imageFive = imageFive;
        }


        const isExists = await this.repository.find({isDeleted: false});
        if(Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if(!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        }
        else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if(!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<StylePageSectionOneCms[] | Error> {
        const data = await this.repository.findAll();
        if(!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Style page section one cms data fetched successfully", "data", data);
    }
}
