import { Controller, Get, Post, Body, HttpCode, UseGuards, HttpStatus, UseInterceptors, UploadedFiles } from '@nestjs/common';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { getMulterConfig } from 'src/core/multer.config';
import { Constants } from 'src/utils/constants';
import { StylePageSectionThreeCmsService } from './style-page-section-three-cms.service';
import { UpdateStylePageSectionThreeCmsDto } from './dto/update-style-page-section-three-cms.dto';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';

@Controller({ path: 'style-page-sec-three-cms', version: Constants.API_VERSION_1 })

export class StylePageSectionThreeCmsController {
    constructor(private readonly service: StylePageSectionThreeCmsService) { }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'image', maxCount: 1 },
            { name: 'imageTwo', maxCount: 1 },
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdateStylePageSectionThreeCmsDto,
        @UploadedFiles() files: { 
            image: UploadedMulterFileI,
            imageTwo: UploadedMulterFileI,
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
