import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from '@nestjs/common';
import { Constants } from 'src/utils/constants';
import { ContactAddressService } from './contact-address.service';
import { CreateContactAddressDto } from './dto/create-contact-address.dto';
import { UpdateContactAddressDto } from './dto/update-contact-address.dto';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';

@Controller({ path: 'contact-address', version: Constants.API_VERSION_1 })

export class ContactAddressController {
    constructor(private readonly service: ContactAddressService) { }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateContactAddressDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateContactAddressDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
