import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateContactMessageDto } from "./dto/create-contact-message.dto";
import { UpdateContactMessageDto } from "./dto/update-contact-message.dto";
import { ContactMessage } from "./entities/contact-message.entity";

@EntityRepository(ContactMessage)
export class ContactMessageRepository extends Repository<ContactMessage> {
    async createEntity(dto: CreateContactMessageDto): Promise<ContactMessage | Error> {
        try {
            const { name, email, phone, website, message, method } = dto;
            const entity = this.create({
                name,
                website,
                email,
                phone,
                message,
                method
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ContactMessage | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateContactMessageDto): Promise<ContactMessage | Error> {
        try {
            await this.update(id, {
                name : dto.name,
                website : dto.website,
                email : dto.email,
                phone : dto.phone,
                message : dto.message,
                method  : dto.method
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query : any): Promise<ContactMessage[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ContactMessage)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if(updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch(e) {
            return e;
        }
    }
}