import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateGlossaryDto } from "./dto/create-glossary.dto";
import { UpdateGlossaryDto } from "./dto/update-glossary.dto";
import { Glossary } from "./entities/glossary.entity";

@EntityRepository(Glossary)
export class GlossaryRepository extends Repository<Glossary> {
    async createEntity(dto: CreateGlossaryDto): Promise<Glossary | Error> {
        try {
            const entity = this.create({
                title: dto.title,
                description: dto.description,
                category: dto.category
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<Glossary | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateGlossaryDto): Promise<Glossary | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                description: dto.description,
                category: dto.category
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<Glossary[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Glossary)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }
}