import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CustomerReviewRepository } from './customer-review.repository';
import { CreateCustomerReviewDto } from './dto/create-customer-review.dto';
import { UpdateCustomerReviewDto } from './dto/update-customer-review.dto';
import { CustomerReview } from './entities/customer-review.entity';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';

@Injectable()
export class CustomerReviewService {
    constructor(
        @InjectRepository(CustomerReviewRepository)
        private readonly repository: CustomerReviewRepository,
        private readonly spaceService: DoSpacesService
    ) { }

    async create(
        dto: CreateCustomerReviewDto,
        files: { 
            reviewerImage: UploadedMulterFileI
        }
    ): Promise<CustomerReview | Error> {
        if (files && files.reviewerImage) {
            const reviewerImage: any = await this.spaceService.uploadFile(files.reviewerImage[0], 'CustomerReview/');
            dto.reviewerImage = reviewerImage;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Customer review data created successfully", "data", data);
    }

    async update(
        id: string,
        dto: UpdateCustomerReviewDto,
        files: { 
            reviewerImage: UploadedMulterFileI
        }
    ): Promise<CustomerReview | Error> {
        const res : any = await this.repository.findOneEntity(id);
        if(!res){
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.reviewerImage) {
            const reviewerImage: any = await this.spaceService.uploadFile(files.reviewerImage[0], 'CustomerReview/');
            dto.reviewerImage = reviewerImage;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Customer review data updated successfully", "data", data);
    }

    async findAll(): Promise<CustomerReview[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Customer review data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<CustomerReview[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Customer review data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Customer review data deleted successfully");
    }
}
