import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateBraceletsSizeDto } from "./dto/create-bracelets-size.dto";
import { UpdateBraceletsSizeDto } from "./dto/update-bracelets-size.dto";
import { BraceletsSize } from "./entities/bracelets-size.entity";

@EntityRepository(BraceletsSize)
export class BraceletsSizeRepository extends Repository<BraceletsSize> {
    async createEntity(dto: CreateBraceletsSizeDto): Promise<BraceletsSize | Error> {
        try {
            const { size, circumference } = dto;
            const entity = this.create({
                size,
                circumference
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<BraceletsSize | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateBraceletsSizeDto): Promise<BraceletsSize | Error> {
        try {
            await this.update(id, {
                size : dto.size,
                circumference : dto.circumference
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query : any): Promise<BraceletsSize[] | Error> {
        try {
            return await this.find({
                order: {createdAt: "DESC"},
                where: { 
                    ...query, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(BraceletsSize)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if(updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch(e) {
            return e;
        }
    }
}