import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard'
import { Constants } from 'src/utils/constants';
import { RingSizeService } from './ring-size.service';
import { CreateRingSizeDto } from './dto/create-ring-size.dto';
import { UpdateRingSizeDto } from './dto/update-ring-size.dto';

@Controller({ path: 'ring-size', version: Constants.API_VERSION_1 })

export class RingSizeController {
    constructor(private readonly service: RingSizeService) { }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(
        @Body() dto: CreateRingSizeDto
    ) {
        return this.service.create(dto);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    update(
        @Param('id') id: string,
        @Body() dto: UpdateRingSizeDto
    ) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
