import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductHighlightVariantsDto } from "./dto/create-product-highlight-variants.dto";
import { UpdateProductHighlightVariantsDto } from "./dto/update-product-highlight-variants.dto";
import { ProductHighlightVariants } from "./entities/product-highlight-variants.entity";

@EntityRepository(ProductHighlightVariants)
export class ProductHighlightVariantsRepository extends Repository<ProductHighlightVariants> {

    async createEntity(dto: CreateProductHighlightVariantsDto): Promise<ProductHighlightVariants | Error> {
        try {
            const { productId, productName, highlightId, highlightName } = dto;
            const entity = this.create({
                productId,
                productName,
                highlightId,
                highlightName
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductHighlightVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductHighlightVariants | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductHighlightVariantsDto): Promise<ProductHighlightVariants | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductHighlightVariants)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductHighlightVariants[] | Error> {
        try {
            return await this.find({
                where: { 
                    name: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteHighlightVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductHighlightVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertHighlightVariant(newHighlight: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductHighlightVariants)
                .values(newHighlight)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductHighlightValues(id: any, newHighlightEntity: any): Promise<any | Error> {
        try {
            const deleteCurrent = await this.deleteHighlightVariant(id);
            await this.bulkInsertHighlightVariant(newHighlightEntity)
        } catch (e) {
            return e;
        }
    }
}