import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { ProductHighlightService } from './product-highlight.service';
import { CreateProductHighlightDto } from './dto/create-product-highlight.dto';
import { UpdateProductHighlightDto } from './dto/update-product-highlight.dto';

@Controller({ path: 'product-highlight', version: Constants.API_VERSION_1 })
export class ProductHighlightController {
    constructor(private readonly service: ProductHighlightService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateProductHighlightDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateProductHighlightDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
