import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { ProductMaterialVariantsRepository } from './product-material-variants.repository';
import { CreateProductMaterialVariantsDto } from './dto/create-product-material-variants.dto';
import { UpdateProductMaterialVariantsDto } from './dto/update-product-material-variants.dto';
import { ProductMaterialVariants } from './entities/product-material-variants.entity';

@Injectable()
export class ProductMaterialVariantsService {
    constructor(
        @InjectRepository(ProductMaterialVariantsRepository)
        private readonly repository: ProductMaterialVariantsRepository
    ) { }

    async create(dto: CreateProductMaterialVariantsDto): Promise<ProductMaterialVariants | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Product material variants created successfully", "data", data);
    }

    async findAll(): Promise<ProductMaterialVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product material variants fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateProductMaterialVariantsDto): Promise<ProductMaterialVariants | Error> {
        const ProductMaterialVariants: any = await this.repository.findOneEntity(id);
        if (!ProductMaterialVariants) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product material variants updated successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductMaterialVariants[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product material variants fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Product material variants deleted successfully");
    }
}
