import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from '@nestjs/common';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { Constants } from 'src/utils/constants';
import { ProductMaterialService } from './product-material.service';
import { CreateProductMaterialDto } from './dto/create-product-material.dto';
import { UpdateProductMaterialDto } from './dto/update-product-material.dto';

@Controller({ path: 'product-material', version: Constants.API_VERSION_1 })
export class ProductMaterialController {
    constructor(private readonly service: ProductMaterialService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateProductMaterialDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateProductMaterialDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
