import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductMaterialDto } from "./dto/create-product-material.dto";
import { UpdateProductMaterialDto } from "./dto/update-product-material.dto";
import { ProductMaterial } from "./entities/product-material.entity";

@EntityRepository(ProductMaterial)
export class ProductMaterialRepository extends Repository<ProductMaterial> {

    async createEntity(dto: CreateProductMaterialDto): Promise<ProductMaterial | Error> {
        try {
            const { name } = dto;
            const entity = this.create({
                name
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductMaterial[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductMaterial | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductMaterialDto): Promise<ProductMaterial | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductMaterial)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductMaterial[] | Error> {
        try {
            return await this.find({
                where: { 
                    name: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }
}