import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductParentCategoryDto } from "./dto/create-product-parent-category.dto";
import { UpdateProductParentCategoryDto } from "./dto/update-product-parent-category.dto";
import { ProductParentCategory } from "./entities/product-parent-category.entity";

@EntityRepository(ProductParentCategory)
export class ProductParentCategoryRepository extends Repository<ProductParentCategory> {

    async createEntity(dto: CreateProductParentCategoryDto): Promise<ProductParentCategory | Error> {
        try {
            const { name, image } = dto;
            const entity = this.create({
                name,
                image
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductParentCategory[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductParentCategory | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductParentCategoryDto): Promise<ProductParentCategory | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                image: dto.image
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductParentCategory)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductParentCategory[] | Error> {
        try {
            return await this.find({
                where: { 
                    name: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findAllEntitiesSelect(query: any): Promise<ProductParentCategory[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }
}