import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductTagDto } from "./dto/create-product-tag.dto";
import { UpdateProductTagDto } from "./dto/update-product-tag.dto";
import { ProductTag } from "./entities/product-tag.entity";

@EntityRepository(ProductTag)
export class ProductTagRepository extends Repository<ProductTag> {

    async createEntity(dto: CreateProductTagDto): Promise<ProductTag | Error> {
        try {
            const { name, image } = dto;
            const entity = this.create({
                name,
                image
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductTag[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductTag | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductTagDto): Promise<ProductTag | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                image: dto.image
            });
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductTag)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductTag[] | Error> {
        try {
            return await this.find({
                where: { 
                    name: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }
}