import { Between, EntityRepository, getConnection, getManager, getRepository, Repository } from "typeorm";
import { Product } from "./entity/product.entity";


@EntityRepository(Product)
export class ProductsRepository extends Repository<Product> {

    async createEntity(
        dto: any
    ): Promise<any | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);

            return entity;
        }
        catch (e) {
            return e;
        }
    }


    async updateEntity(
        id: string,
        dto: any
    ): Promise<Product | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }



    async findAllEntities(take: number, page: number): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    take, skip: take * (page - 1),
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false },
                    relations: [
                        'productParentCategory',
                        'productsubCategory',
                        'productChildCategory',
                        'productImages',
                        'productColorVariants',
                        'productSizeVariants',
                        'productTagVariants',
                        'productMaterialVariants',
                        'productHighlightVariants'
                    ]
                }
            );
        }
        catch (e) {
            return e;
        }
    }

    async findEntitiesByLimit(count: number): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    take: count,
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false },
                    relations: ['photos', 'productsubCategory', 'productParentCategory'],
                },

            );
        }
        catch (e) {
            return e;
        }
    }

    async findProductBySubCategoryRepo(categoryId: string, take: number, page: number): Promise<any | Error> {
        try {
            const data = await this.find(
                {
                    take, skip: take * (page - 1),
                    order: { createdAt: "DESC" },
                    where: { secondCategoryId: categoryId, isDeleted: false },
                    relations: ['photos', 'category', 'thicknesses'],
                },

            );

            const count = await this.count(
                {
                    order: { createdAt: "DESC" },
                    where: { secondCategoryId: categoryId, isDeleted: false },
                    relations: ['photos', 'category', 'thicknesses'],
                },

            );

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        }
        catch (e) {
            return e;
        }
    }


    async findProductByBrand(brandName: string, take: number, page: number, categoryId: String,
        maxPrice: number,
        minPrice: number,
        thickness: any, sortBy: any): Promise<any | Error> {
        try {

            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.firstCategoryId = '${categoryId}'`;
            }
            if (brandName) {
                //console.log(brandName);
                if (whereQuery.length > 3) {
                    whereQuery += `and p.brandName = '${brandName}'`;
                }
                else {
                    whereQuery += ` p.brandName = '${brandName}'`;
                }

            }
            if (maxPrice && minPrice) {
                whereQuery += `and (p.price >= ${minPrice} and p.price <= ${maxPrice})`;
            }
            // if (color) {
            //     var result = '(';
            //     if (color.length > 0) {
            //         color = color.split(",")
            //     }
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += ` and c.color IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` c.color IN ${result} `;
            //     }

            // }
            // if (size) {
            //     var result = '(';
            //     if (size.length > 0) {
            //         size = size.split(",")
            //     }
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += `and s.size IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` s.size IN ${result} `;
            //     }

            // }
            if (thickness) {
                var result = '(';
                if (thickness.length > 0) {
                    thickness = thickness.split(",")
                }
                thickness.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.thickness IN ${result} `;
                }
                else {
                    whereQuery += ` t.thickness IN ${result} `;
                }

            }

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 `;
            }
            else {
                whereQuery += ` p.isDeleted=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            }
            else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }


            let sql = `SELECT p.* FROM products as p 
                        LEFT JOIN thickness_variant as t on t.productId=p.id
                        ${conditionQuery} GROUP BY p.id limit ${offset}, ${take};`
            let sqlTotal = `SELECT p.* FROM products as p 
                            LEFT JOIN thickness_variant as t on t.productId=p.id
                            ${conditionQuery} group by p.id;`
            //console.log(sql);
            //console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        }
        catch (e) {
            return e;
        }
    }

    async findProductByMenu(routeName: string, take: number, page: number, categoryId: String,
        brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        try {
            if (routeName === "new_arrival") {
                //console.log(take);
                //console.log(page);
                let whereQuery: any = ``;
                if (categoryId) {
                    whereQuery = `p.firstCategoryId = '${categoryId}' `;
                }
                if (brand) {
                    var result = '(';
                    if (brand.length > 0) {
                        brand = brand.split(",")
                    }
                    //console.log(brand);
                    brand.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and p.brandName IN ${result} `;
                    }
                    else {
                        whereQuery += ` p.brandName IN ${result} `;
                    }

                }
                if (maxPrice && minPrice) {
                    if (whereQuery.length > 3) {
                        whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    }
                    else {
                        whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    }

                }
                // if (color) {
                //     var result = '(';
                //     if (color.length > 0) {
                //         color = color.split(",")
                //     }
                //     color.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += ` and c.color IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` c.color IN ${result} `;
                //     }

                // }
                // if (size) {
                //     var result = '(';
                //     if (size.length > 0) {
                //         size = size.split(",")
                //     }
                //     size.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += `and s.size IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` s.size IN ${result} `;
                //     }

                // }
                if (thickness) {
                    var result = '(';
                    if (thickness.length > 0) {
                        thickness = thickness.split(",")
                    }
                    thickness.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and t.thickness IN ${result} `;
                    }
                    else {
                        whereQuery += ` t.thickness IN ${result} `;
                    }

                }

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.isDeleted=0 `;
                }
                else {
                    whereQuery += ` p.isDeleted=0 `;
                }

                if (whereQuery.substring(0, 3) === 'and') {
                    whereQuery = whereQuery.slice(4, whereQuery.length);
                }

                let conditionQuery = ``;

                if (!whereQuery) {
                    conditionQuery = ``;
                }
                else {
                    conditionQuery = ` WHERE ${whereQuery} `;
                }

                let offset = 0;


                if (page > 1) {
                    offset = ((page - 1) * take);
                }


                let sql = `SELECT p.* FROM new_arrival as p 
                            LEFT JOIN thickness_variant as t on t.productId=p.id
                             ${conditionQuery} GROUP BY p.id limit ${offset}, ${take};`
                let sqlTotal = `SELECT p.* FROM new_arrival as p 
                                  LEFT JOIN thickness_variant as t on t.productId=p.id
                                  ${conditionQuery} group by p.id;`
                //console.log(sql);
                //console.log(sqlTotal);
                const entityManager = getManager();
                const data = await entityManager.query(sql);
                const totalData = await entityManager.query(sqlTotal);
                const count = totalData.length;


                //console.log('Repo = ', routeName);
                const pageCount = Math.ceil(count / take);

                const responseData = {
                    page: page,
                    limit: take,
                    pageCount: pageCount,
                    totalDataFound: count,
                    data
                };

                return responseData;
            }
            if (routeName === "best_selling") {
                //console.log(take);
                //console.log(page);
                let whereQuery: any = ``;
                if (categoryId) {
                    whereQuery = `p.firstCategoryId = '${categoryId}' `;
                }
                if (brand) {
                    var result = '(';
                    if (brand.length > 0) {
                        brand = brand.split(",")
                    }
                    //console.log(brand);
                    brand.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and p.brandName IN ${result} `;
                    }
                    else {
                        whereQuery += ` p.brandName IN ${result} `;
                    }

                }
                if (maxPrice && minPrice) {
                    if (whereQuery.length > 3) {
                        whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    }
                    else {
                        whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    }

                }
                // if (color) {
                //     var result = '(';
                //     if (color.length > 0) {
                //         color = color.split(",")
                //     }
                //     color.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += ` and c.color IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` c.color IN ${result} `;
                //     }

                // }
                // if (size) {
                //     var result = '(';
                //     if (size.length > 0) {
                //         size = size.split(",")
                //     }
                //     size.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += `and s.size IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` s.size IN ${result} `;
                //     }

                // }
                if (thickness) {
                    var result = '(';
                    if (thickness.length > 0) {
                        thickness = thickness.split(",")
                    }
                    thickness.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and t.thickness IN ${result} `;
                    }
                    else {
                        whereQuery += ` t.thickness IN ${result} `;
                    }

                }

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.isDeleted=0 `;
                }
                else {
                    whereQuery += ` p.isDeleted=0 `;
                }

                if (whereQuery.substring(0, 3) === 'and') {
                    whereQuery = whereQuery.slice(4, whereQuery.length);
                }

                let conditionQuery = ``;

                if (!whereQuery) {
                    conditionQuery = ``;
                }
                else {
                    conditionQuery = ` WHERE ${whereQuery} `;
                }

                let offset = 0;


                if (page > 1) {
                    offset = ((page - 1) * take);
                }



                let sql = `SELECT p.* FROM best_selling as p 
                            LEFT JOIN thickness_variant as t on t.productId=p.id
                            ${conditionQuery} group by p.id  limit ${offset}, ${take};`
                let sqlTotal = `SELECT p.* FROM best_selling as p 
                                  LEFT JOIN thickness_variant as t on t.productId=p.id
                                  ${conditionQuery} group by p.id;`
                //console.log(sql);
                //console.log(sqlTotal);
                const entityManager = getManager();
                const data = await entityManager.query(sql);
                const totalData = await entityManager.query(sqlTotal);
                const count = totalData.length;


                //console.log('Repo = ', routeName);
                const pageCount = Math.ceil(count / take);

                const responseData = {
                    page: page,
                    limit: take,
                    pageCount: pageCount,
                    totalDataFound: count,
                    data
                };

                return responseData;
            }
        }
        catch (e) {
            return e;
        }
    }

    async findProductByChildCategory(categoryId: string, take: number, page: number, brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        try {
            //console.log(take);
            //console.log(page);
            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.thirdCategoryId = '${categoryId}' `;
            }
            if (brand) {
                var result = '(';
                if (brand.length > 0) {
                    brand = brand.split(",")
                }
                //console.log(brand);
                brand.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.brandName IN ${result} `;
                }
                else {
                    whereQuery += ` p.brandName IN ${result} `;
                }

            }
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
                else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }

            }
            // if (color) {
            //     var result = '(';
            //     if (color.length > 0) {
            //         color = color.split(",")
            //     }
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += ` and c.color IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` c.color IN ${result} `;
            //     }

            // }
            // if (size) {
            //     var result = '(';
            //     if (size.length > 0) {
            //         size = size.split(",")
            //     }
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += `and s.size IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` s.size IN ${result} `;
            //     }

            // }
            if (thickness) {
                var result = '(';
                if (thickness.length > 0) {
                    thickness = thickness.split(",")
                }
                thickness.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.thickness IN ${result} `;
                }
                else {
                    whereQuery += ` t.thickness IN ${result} `;
                }

            }

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 `;
            }
            else {
                whereQuery += ` p.isDeleted=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            }
            else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }



            let sql = `SELECT p.* FROM products as p 
                        LEFT JOIN color_variant as c on c.productId=p.id
                        LEFT JOIN size_variant as s on s.productId=p.id
                        LEFT JOIN thickness_variant as t on t.productId=p.id
                        ${conditionQuery} group by p.id  limit ${offset}, ${take}; `
            let sqlTotal = `SELECT p.* FROM products as p 
                                LEFT JOIN color_variant as c on c.productId=p.id
                                LEFT JOIN size_variant as s on s.productId=p.id
                                LEFT JOIN thickness_variant as t on t.productId=p.id
                                ${conditionQuery} group by p.id;`
            //console.log(sql);
            //console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        }
        catch (e) {
            return e;
        }
    }

    async findProductByChildCategoryRepo(categoryId: string, take: number, page: number): Promise<any | Error> {
        try {
            const data = await this.find(
                {
                    take, skip: take * (page - 1),
                    order: { createdAt: "DESC" },
                    where: { thirdCategoryId: categoryId, isDeleted: false },
                    relations: ['photos', 'category', 'thicknesses'],
                },
            );

            const count = await this.count(
                {
                    order: { createdAt: "DESC" },
                    where: { thirdCategoryId: categoryId, isDeleted: false },
                    relations: ['photos', 'category', 'thicknesses'],
                },
            );

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;




        }
        catch (e) {
            return e;
        }
    }

    async findProductByParentCategory(categoryId: string, take: number, page: number, brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        try {

            //console.log(take);
            //console.log(page);
            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.firstCategoryId = '${categoryId}' `;
            }
            if (brand) {
                var result = '(';
                if (brand.length > 0) {
                    brand = brand.split(",")
                }
                //console.log(brand);
                brand.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.brandName IN ${result} `;
                }
                else {
                    whereQuery += ` p.brandName IN ${result} `;
                }

            }
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
                else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }

            }
            // if (color) {
            //     var result = '(';
            //     if (color.length > 0) {
            //         color = color.split(",")
            //     }
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += ` and c.color IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` c.color IN ${result} `;
            //     }

            // }
            // if (size) {
            //     var result = '(';
            //     if (size.length > 0) {
            //         size = size.split(",")
            //     }
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += `and s.size IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` s.size IN ${result} `;
            //     }

            // }
            if (thickness) {
                var result = '(';
                if (thickness.length > 0) {
                    thickness = thickness.split(",")
                }
                thickness.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.thickness IN ${result} `;
                }
                else {
                    whereQuery += ` t.thickness IN ${result} `;
                }

            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 `;
            }
            else {
                whereQuery += ` p.isDeleted=0 `;
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            }
            else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }



            let sql = `SELECT p.* FROM products as p 
                        LEFT JOIN thickness_variant as t on t.productId=p.id
                        ${conditionQuery}  GROUP BY p.id limit ${offset}, ${take};`
            let sqlTotal = `SELECT p.* FROM products as p 
                                LEFT JOIN thickness_variant as t on t.productId=p.id
                                ${conditionQuery} group by p.id;`
            //console.log(sql);
            //console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;
        }
        catch (e) {
            return e;
        }
    }

    async findProductByParentCategoryRepo(categoryId: string, take: number, page: number): Promise<any | Error> {
        try {



            const data = await this.find(
                {
                    take, skip: take * (page - 1),
                    order: { createdAt: "DESC" },
                    where: { firstCategoryId: categoryId, isDeleted: false },
                    relations: ['photos', 'category', 'thicknesses'],
                },

            );

            const count = await this.count(
                {
                    order: { createdAt: "DESC" },
                    where: { firstCategoryId: categoryId, isDeleted: false },
                    relations: ['photos', 'category', 'thicknesses'],
                },

            );

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;
        }
        catch (e) {
            return e;
        }
    }

    async findFeaturedProduct(count: number): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    take: count,
                    order: { createdAt: "DESC" },
                    where: { isFeatured: true, isDeleted: false },
                    relations: ['photos', 'productsubCategory', 'productParentCategory'],
                },

            );
        }
        catch (e) {
            return e;
        }
    }


    async getProductsByQuery(productIds: any): Promise<Product[] | Error> {
        try {
            const products = await getConnection()
                .createQueryBuilder()
                .from(Product, "product")
                .where("product.id IN (:...ids)", { ids: productIds })
                .execute();

            return products;
        }
        catch (e) {
            return e;
        }
    }


    async findOneRawEntity(
        id: string
    ): Promise<any | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return false;
        }
    }

    async findOneEntity(id: string): Promise<any | Error> {
        let query: any = id;
        query.isDeleted = false;
        console.log(query);
        try {
            return await this.find({
                where: query,
                relations: [
                    'productParentCategory',
                    'productsubCategory',
                    'productChildCategory',
                    'productImages',
                    'productColorVariants',
                    'productSizeVariants',
                    'productTagVariants',
                    'productMaterialVariants',
                    'productHighlightVariants'
                ]
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneProduct(
        id: string
    ): Promise<Product[] | Error> {
        let query: any = { id: id };
        query.isDeleted = false;
        try {
            return await this.find({
                where: query,
                relations: ['photos', 'productsubCategory', 'productParentCategory'],
            },
            );
        }
        catch (e) {
            return e;
        }
    }


    async findByFeaturedProduct(
    ): Promise<Product[] | Error> {
        try {
            return await this.find({
                where: { isFeatured: true, isDeleted: false },
                relations: ['photos', 'productsubCategory', 'productParentCategory'],
            },
            );
        }
        catch (e) {
            return e;
        }
    }



    async productsFilterByParams(
        limit: number,
        page: number,
        categoryId: String,
        brandName: any,
        maxPrice: number,
        minPrice: number,
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.firstCategoryId = '${categoryId}'`;
            }
            if (brandName) {
                var result = '(';
                brandName.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                whereQuery += `and p.brandName IN ${result}`;
            }
            if (maxPrice && minPrice) {
                whereQuery += `and (p.price >= ${minPrice} and p.price <= ${maxPrice})`;
            }
            // if (color) {
            //     var result = '(';
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     whereQuery += `and c.color IN ${result}`;
            // }
            // if (size) {
            //     var result = '(';
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     whereQuery += `and s.size IN ${result}`;
            // }
            // if (thickness) {
            //     var result = '(';
            //     thickness.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     whereQuery += `and t.thickness IN ${result}`;
            // }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            if (!whereQuery) {
                whereQuery = `1`;
            }

            let sql = `SELECT p.id, p.name, p.price, p.brandName, p.featuredImage
                        FROM products p
                        WHERE ${whereQuery}
                        ORDER BY p.name;`

            const entityManager = getManager();
            const query = await entityManager.query(sql);

            return query;
        }
        catch (e) {
            return e;
        }
    }

    async filterByProductsName(
        take: any,
        page: any,
        keyword: any,
        routeName: any,
        parentCategoryId: any,
        subCategoryId: any,
        childCategoryId: any,
        maxPrice: any,
        minPrice: any,
        color: any,
        size: any,
        tag: any,
        material: any,
        highlight: any,
        sortBy: any): Promise<any | Error> {
        try {
            // console.log(take, "page=", page, "keyword=", keyword, "routeName=", routeName, "parentCategoryId=", parentCategoryId, "subCategoryId=", subCategoryId, "childCategoryId=", childCategoryId, "maxPrice=", maxPrice, "minPrice=", minPrice, "color=", color, "size=", size, "tag=", tag, "material=", material, "highlight=", highlight, "sortBy=", sortBy);
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            // sortBy operation
            if (sortBy === 'new') {
                orderByQuery = `ORDER BY p.createdAt DESC `;
            }

            if (sortBy === 'high') {
                orderByQuery = `ORDER BY p.price DESC`;
            }

            if (sortBy === 'low') {
                orderByQuery = `ORDER BY p.price ASC`;
            }

            // routeName operation
            if (routeName === 'merch') {
                whereQuery = `p.isMerch = 1 `;
            }

            if (routeName === 'giftGuide') {
                if(whereQuery.length > 3){
                    whereQuery += ` and p.isGiftGuide = 1 `;
                }
                else{
                    whereQuery = `p.isGiftGuide = 1 `;
                }
            }


            // keyword operation searh by name and child category name and parent category name
            if (keyword) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%' OR 
                    p.childCategoryName LIKE '%${keyword}%' `;
                }
                else {
                    whereQuery += ` p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%' OR 
                    p.childCategoryName LIKE '%${keyword}%' `;
                }
            }

            // filter using parent category id
            if (parentCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.parentCategoryId = '${parentCategoryId}' `;
                }
                else {
                    whereQuery += ` p.parentCategoryId = '${parentCategoryId}' `;
                }

            }

            // filter using sub category id
            if (subCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.subCategoryId = '${subCategoryId}' `;
                }
                else {
                    whereQuery += ` p.subCategoryId = '${subCategoryId}' `;
                }

            }

            // filter using child category id
            if (childCategoryId) {

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.childCategoryId = '${childCategoryId}' `;
                }
                else {
                    whereQuery += ` p.childCategoryId = '${childCategoryId}' `;
                }

            }

            // filter using max price and min price
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
                else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
            }

            // filter using color
            if (color) {
                var result = '(';
                if (color.length > 0) {
                    color = color.split(",")
                }
                color.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and pcv.colorName IN ${result} `;
                }
                else {
                    whereQuery += ` pcv.colorName IN ${result} `;
                }

            }

            // filter using size
            if (size) {
                // console.log("initial size", size)
                var result = '(';
                if (size.length > 0) {
                    size = size.split(",")
                }
                size.forEach(item => {
                    // console.log("item", item)
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and psv.sizeName IN ${result} `;
                }
                else {
                    whereQuery += ` psv.sizeName IN ${result} `;
                }

            }

            // filter using tag
            if (tag) {
                var result = '(';
                if (tag.length > 0) {
                    tag = tag.split(",")
                }
                tag.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and ptv.tagName IN ${result} `;
                }
                else {
                    whereQuery += ` ptv.tagName IN ${result} `;
                }

            }

            // filter using material
            if (material) {
                var result = '(';
                if (material.length > 0) {
                    material = material.split(",")
                }
                material.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and pmv.materialName IN ${result} `;
                }
                else {
                    whereQuery += ` pmv.materialName IN ${result} `;
                }

            }

            // filter using highlight
            if (highlight) {
                var result = '(';
                if (highlight.length > 0) {
                    highlight = highlight.split(",")
                }
                highlight.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and phv.highlightName IN ${result} `;
                }
                else {
                    whereQuery += ` phv.highlightName IN ${result} `;
                }

            }



            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 and p.isDeactive=0 `;
            }
            else {
                whereQuery += ` p.isDeleted=0 and p.isDeactive=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            }
            else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }


            let sql = `SELECT p.*, (SELECT GROUP_CONCAT(CONCAT(colorCode)) AS colorCode FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode  FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            ${conditionQuery} ${orderByQuery} GROUP BY p.id limit ${offset}, ${take};`

            let sqlTotal = `SELECT p.* FROM products as p
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId 
            ${conditionQuery}  GROUP BY p.id ;`
            console.log(sql);
            //console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        }
        catch (e) {
            return e;
        }
    }


    async discountedProductWithColor(
        take: any,
        page: any): Promise<any | Error> {
        try {
            // console.log(take, "page=", page, "keyword=", keyword, "routeName=", routeName, "parentCategoryId=", parentCategoryId, "subCategoryId=", subCategoryId, "childCategoryId=", childCategoryId, "maxPrice=", maxPrice, "minPrice=", minPrice, "color=", color, "size=", size, "tag=", tag, "material=", material, "highlight=", highlight, "sortBy=", sortBy);
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            // sortBy operation

            // filter using parent category id
            whereQuery += ` p.discountPrice > 0 AND p.isDeleted=0`;
            let conditionQuery = ``;
                conditionQuery = ` WHERE ${whereQuery} `;
            let offset = 0;
            if (page > 1) {
                offset = ((page - 1) * take);
            }

            orderByQuery = `ORDER BY p.createdAt DESC `;


            let sql = `SELECT p.*, (SELECT GROUP_CONCAT(CONCAT(colorCode)) AS colorCode FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode  FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            ${conditionQuery} GROUP BY p.id ${orderByQuery}  limit ${offset}, ${take};`
            console.log(sql);
            //console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);

            return data;

        }
        catch (e) {
            return [];
        }
    }

    async discountedProductWithColorCount(): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            whereQuery += ` p.discountPrice > 0 AND p.isDeleted=0`;
            let conditionQuery = ``;
                conditionQuery = ` WHERE ${whereQuery} `;

            let sqlTotal = `SELECT p.* FROM products as p
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId 
            ${conditionQuery} GROUP BY p.id ;`
            const entityManager = getManager();
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            console.log('Count = ', count);
            return count;

        }
        catch (e) {
            return e;
        }
    }

    async getProductByCategory(categoryId: string): Promise<any | Error> {
        try {

            return await getRepository(Product)
                .createQueryBuilder()
                .select("Product.id")
                .where("firstCategoryId = :categoryId OR secondCategoryId = :categoryId OR thirdCategoryId = :categoryId", {
                    categoryId: categoryId
                })
                .getMany();
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Product)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async countProduct(): Promise<any | Error> {
        try {
            return await this.count({
                where: { isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async todayProducts(): Promise<any | Error> {
        const start = new Date();
        start.setUTCHours(0, 0, 0, 0);
        const end = new Date();
        end.setUTCHours(23, 59, 59, 999);

        const data = await this.find({
            where: {
                isDeleted: false,
                createdAt: Between(
                    new Date(start).toISOString(),
                    new Date(end).toISOString()
                )
            },
            relations: [
                'productParentCategory',
                'productsubCategory',
                'productChildCategory',
                'productImages',
                'productColorVariants',
                'productSizeVariants',
                'productTagVariants',
                'productMaterialVariants',
                'productHighlightVariants'
            ]
        });

        return data;
    }

    async bestSelling(): Promise<any | Error> {
        const sql = `SELECT p.* FROM best_selling as p WHERE p.isDeleted=false ORDER BY p.soldQuantity DESC LIMIT 8;`
        const entityManager = getManager();
        const data = await entityManager.query(sql);
        return data;
    }

    async filterRelatedProductEntities(subCategoryId: string): Promise<any | Error> {
        try {
            return await this.find({
                take: 9,
                order: { createdAt: -1 },
                where: [{ subCategoryId: subCategoryId, isDeleted: false }],
                relations: ['photos', 'productsubCategory', 'productParentCategory']
            });
        }
        catch (e) {
            return e;
        }
    }

    async exportAllProductRepo(): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false },
                    relations: ['photos', 'productsubCategory', 'productParentCategory']
                },

            );
        }
        catch (e) {
            return e;
        }
    }


    async productReorderRepository(
        limit: number,
        page: number,
        categoryId: String
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;


            whereQuery = ` p.stock <= p.quantityReorder `;

            whereQuery += ` AND p.isDeleted=0 `;

            if (categoryId) {
                whereQuery += ` AND p.subCategoryId = '${categoryId}'`;
            }


            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            }
            else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * limit);
            }


            let sql = `SELECT p.* FROM products as p 
                        ${conditionQuery} GROUP BY p.id;`
            let sqlTotal = `SELECT p.* FROM products as p 
                                ${conditionQuery} group by p.id;`
            //console.log(sql);
            //console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / limit);

            const responseData = {
                page: page,
                limit: limit,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;


        }
        catch (e) {
            return e;
        }
    }

    async topProducts(): Promise<any | Error> {
        try {
            const productData = await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    totalOrder: -1
                },
                take: 3
            });
            let labelArr = [];
            let dataArr = [];
            let dataArray = [];
            for (let i = 0; i < productData.length; i++) {
                const data = productData[i];
                labelArr.push(data.name);
                dataArr.push(data.totalOrder);
                // dataArray.push(arr);
                // arr = [];

            }
            return {
                labels: labelArr,
                data: dataArr
            };
        }
        catch (e) {
            return e;
        }
    }

    async topTwoProducts(): Promise<any | Error> {
        try {
            const productData = await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    totalOrder: -1
                },
                take: 2
            });
            // let labelArr = [];
            // let dataArr = [];
            // let dataArray = [];
            // for (let i = 0; i < productData.length; i++) {
            //     const data = productData[i];
            //     // labelArr.push(data.name);
            //     // dataArr.push(data.totalOrder);
            //     dataArray.push(data);
            //     // arr = [];

            // }
            // // return {
            // //     labels: labelArr,
            // //     data: dataArr
            // // };
            return productData;
        }
        catch (e) {
            return e;
        }
    }

    async relaventProducts(subCategoryId: string): Promise<any | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false,
                    subCategoryId: subCategoryId
                },
                take: 10
            });
        }
        catch (e) {
            return e;
        }
    }

}