import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { generateStockId, isArraySame } from 'src/helper/common-helper';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateImportProductDto } from './dto/create-import-product.dto';
import { CreateProductDto } from './dto/create-product.dto';
import { UpdateProductDto } from './dto/update-product.dto';
import { Product } from './entity/product.entity';
import { ProductsRepository } from './product.repository';
import { v4 as uuidv4 } from 'uuid';
import readXlsxFile from 'read-excel-file/node';
import { CreateConfirmImportProductDto } from './dto/create-confirm-import-product.dto';
import { CreateConfirmStockInProductDto } from './dto/create-confirm-stockin-product.dto';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { ProductParentCategoryRepository } from '../product-parent-category/product-parent-category.repository';
import { ProductColorRepository } from '../product-color/product-color.repository';
import { ProductColorVariantsRepository } from '../product-color-variants/product-color-variants.repository';
import { CreateProductColorVariantsDto } from '../product-color-variants/dto/create-product-color-variants.dto';
import { ProductSizeRepository } from '../product-size/product-size.repository';
import { ProductSizeVariantsRepository } from '../product-size-variants/product-size-variants.repository';
import { CreateProductSizeVariantsDto } from '../product-size-variants/dto/create-product-size-variants.dto';
import { ProductTagRepository } from '../product-tag/product-tag.repository';
import { ProductTagVariantsRepository } from '../product-tag-variants/product-tag-variants.repository';
import { CreateProductTagVariantsDto } from '../product-tag-variants/dto/create-product-tag-variants.dto';
import { ProductImageGalleryRepository } from '../product-image-gallery/product-image-gallery.repository';
import { CreateProductImageGalleryDto } from '../product-image-gallery/dto/create-product-image-gallery.dto';
import { ProductMaterialRepository } from '../product-material/product-material.repository';
import { ProductMaterialVariantsRepository } from '../product-material-variants/product-material-variants.repository';
import { ProductHighlightRepository } from '../product-highlight/product-highlight.repository';
import { ProductHighlightVariantsRepository } from '../product-highlight-variants/product-highlight-variants.repository';
import { CreateProductMaterialVariantsDto } from '../product-material-variants/dto/create-product-material-variants.dto';
import { CreateProductHighlightVariantsDto } from '../product-highlight-variants/dto/create-product-highlight-variants.dto';
import { ProductStockImportRepository } from '../product-stock-import/product-stock-importrepository';
import { CreateProductStockImportDto } from '../product-stock-import/dto/create-product-stock-import.dto';
import { ProductSubCategoryRepository } from '../product-sub-category/product-sub-category.repository';
import { ProductChildCategoryRepository } from '../product-child-category/product-child-category.repository';
import { CreateProductColorDto } from '../product-color/dto/create-product-color.dto';
import { CreateProductSizeDto } from '../product-size/dto/create-product-size.dto';
import { CreateProductTagDto } from '../product-tag/dto/create-product-tag.dto';
import { CreateProductMaterialDto } from '../product-material/dto/create-product-material.dto';
import { CreateProductHighlightDto } from '../product-highlight/dto/create-product-highlight.dto';
import { ProductDumpImportRepository } from '../product-dump-import/product-dump-importrepository';
import { CreateProductDumpImportDto } from '../product-dump-import/dto/create-product-dump-import.dto';

@Injectable()
export class ProductService {

    constructor(
        @InjectRepository(ProductsRepository)
        private readonly repository: ProductsRepository,
        private readonly doSpaceService: DoSpacesService,
        @InjectRepository(ProductParentCategoryRepository)
        private readonly productParentCatRepo: ProductParentCategoryRepository,
        @InjectRepository(ProductColorRepository)
        private readonly productColorRepo: ProductColorRepository,
        @InjectRepository(ProductColorVariantsRepository)
        private readonly productColorVariantsRepo: ProductColorVariantsRepository,
        @InjectRepository(ProductSizeRepository)
        private readonly productSizeRepo: ProductSizeRepository,
        @InjectRepository(ProductSizeVariantsRepository)
        private readonly productSizeVariantsRepo: ProductSizeVariantsRepository,
        @InjectRepository(ProductTagRepository)
        private readonly productTagRepo: ProductTagRepository,
        @InjectRepository(ProductTagVariantsRepository)
        private readonly productTagVariantsRepo: ProductTagVariantsRepository,
        @InjectRepository(ProductImageGalleryRepository)
        private readonly productImageGalleryRepo: ProductImageGalleryRepository,
        @InjectRepository(ProductMaterialRepository)
        private readonly productMaterialRepo: ProductMaterialRepository,
        @InjectRepository(ProductMaterialVariantsRepository)
        private readonly productMaterialVariantsRepo: ProductMaterialVariantsRepository,
        @InjectRepository(ProductHighlightRepository)
        private readonly productHighlightRepo: ProductHighlightRepository,
        @InjectRepository(ProductHighlightVariantsRepository)
        private readonly productHighlightVariantsRepo: ProductHighlightVariantsRepository,
        @InjectRepository(ProductStockImportRepository)
        private readonly productStockImportRepo: ProductStockImportRepository,
        @InjectRepository(ProductSubCategoryRepository)
        private readonly productSubCatRepo: ProductSubCategoryRepository,
        @InjectRepository(ProductChildCategoryRepository)
        private readonly productChildCatRepo: ProductChildCategoryRepository,
        @InjectRepository(ProductDumpImportRepository)
        private readonly productDumpImportRepo: ProductDumpImportRepository,
    ) { }

    async create(dto: CreateProductDto, files: {
        productImages?: UploadedMulterFileI,
        featuredImage?: UploadedMulterFileI,
    }): Promise<any> {
        // check parent category exist or not
        const productParentCatData = await this.productParentCatRepo.findOne(
            dto.parentCategoryId
        );
        if (!productParentCatData) {
            throw new HttpException(
                "Category not found",
                HttpStatus.NOT_FOUND
            );
        }
        // end of parent category check 

        // process featured image and upload it cloud storage
        if (files && files.featuredImage) {
            const featuredImage: any = await this.doSpaceService.uploadFile(files.featuredImage[0], 'Product/');
            dto.featuredImage = featuredImage;
        } else dto.featuredImage = 'https://golden-gal.nyc3.cdn.digitaloceanspaces.com/default_cms.png';
        // end of featured image process

        // Create a product entity
        const productEntity = {
            name: dto.name,
            featuredImage: dto.featuredImage,
            sku: dto.sku,
            rating: dto.rating,
            price: dto.price,
            cost: dto.cost,
            stock: dto.stock,
            weight: dto.weight ? dto.weight : 0,
            vendorName: dto.vendorName,
            parentCategoryId: dto.parentCategoryId,
            parentCategoryName: dto.parentCategoryName,
            subCategoryId: dto.subCategoryId,
            subCategoryName: dto.subCategoryName,
            childCategoryId: dto.childCategoryId,
            childCategoryName: dto.childCategoryName,
            isFeatured: dto.isFeatured,
            isDeactive: dto.isDeactive?dto.isDeactive:0,
            isMerch: dto.isMerch,
            isGiftGuide: dto.isGiftGuide,
            isReadyToShip: dto.isReadyToShip,
            shortDescription: dto.shortDescription,
            longDescription: dto.longDescription,
        }
        console.log("productEntity=", productEntity)
        const product = await this.repository.createEntity(productEntity as CreateProductDto);
        // end of product create 

        // processiong color and create it into color variants
        if (dto.color && dto.color.length > 0) {
            let colors: any;
            if (typeof dto.color === 'string') {
                colors = JSON.parse(dto.color);
            }
            else colors = dto.color;
            colors.forEach(async (element: any) => {
                const colorData = await this.productColorRepo.findByName(element);
                if (Object.keys(colorData).length > 0) {
                    const colorVariant = {
                        productId: product.id,
                        productName: product.name,
                        colorId: colorData[0]['id'],
                        colorName: element,
                        colorCode: colorData[0]['colorCode']
                    }
                    const productColorVariant = await this.productColorVariantsRepo.createEntity(colorVariant as CreateProductColorVariantsDto)
                }
                else {
                    console.log(`${element} color not found!`)
                }
            });
        }
        // end of product color loop 


        // processiong size and create it into size variants
        if (dto.size && dto.size.length > 0) {
            let sizes: any;
            if (typeof dto.size === 'string') {
                sizes = JSON.parse(dto.size);
            }
            else sizes = dto.size;
            sizes.forEach(async (element: any) => {
                const sizeData = await this.productSizeRepo.findByName(element);
                if (Object.keys(sizeData).length > 0) {
                    const sizeVariant = {
                        productId: product.id,
                        productName: product.name,
                        sizeId: sizeData[0]['id'],
                        sizeName: element
                    }
                    const proSizeVariant = await this.productSizeVariantsRepo.createEntity(sizeVariant as CreateProductSizeVariantsDto)
                }
                else {
                    console.log(`${element} size not found!`)
                }
            });
        }
        // end of product size process

        // processiong tag and create it into tag variants
        if (dto.tag && dto.tag.length > 0) {
            let tags: any;
            if (typeof dto.tag === 'string') {
                tags = JSON.parse(dto.tag);
            }
            else tags = dto.tag;
            tags.forEach(async (element: any) => {
                const tagData = await this.productTagRepo.findByName(element);
                if (Object.keys(tagData).length > 0) {
                    const tagVariant = {
                        productId: product.id,
                        productName: product.name,
                        tagId: tagData[0]['id'],
                        tagName: element
                    }
                    const proTagVariant = await this.productTagVariantsRepo.createEntity(tagVariant as CreateProductTagVariantsDto)
                }
                else {
                    console.log(`${element} tag not found!`)
                }
            });
        }
        // end of product tag process

        // processiong material and create it into material variants
        if (dto.material && dto.material.length > 0) {
            let materials: any;
            if (typeof dto.material === 'string') {
                materials = JSON.parse(dto.material);
            }
            else materials = dto.material;
            materials.forEach(async (element: any) => {
                const materialData = await this.productMaterialRepo.findByName(element);
                if (Object.keys(materialData).length > 0) {
                    const materialVariant = {
                        productId: product.id,
                        productName: product.name,
                        materialId: materialData[0]['id'],
                        materialName: element
                    }
                    const proMaterialVariant = await this.productMaterialVariantsRepo.createEntity(materialVariant as CreateProductMaterialVariantsDto)
                }
                else {
                    console.log(`${element} material not found!`)
                }
            });
        }
        // end of product material process

        // processiong highlight and create it into highlight variants
        if (dto.highlight && dto.highlight.length > 0) {
            let highlights: any;
            if (typeof dto.highlight === 'string') {
                highlights = JSON.parse(dto.highlight);
            }
            else highlights = dto.highlight;
            highlights.forEach(async (element: any) => {
                const highlightData = await this.productHighlightRepo.findByName(element);
                if (Object.keys(highlightData).length > 0) {
                    const highlightVariant = {
                        productId: product.id,
                        productName: product.name,
                        highlightId: highlightData[0]['id'],
                        highlightName: element
                    }
                    const proHighlightVariant = await this.productHighlightVariantsRepo.createEntity(highlightVariant as CreateProductHighlightVariantsDto)
                }
                else {
                    console.log(`${element} highlight not found!`)
                }
            });
        }
        // end of product highlight process

        // processing product image
        if (files && files.productImages) {
            const productFile = files['productImages'];
            Array(productFile).forEach(async (element: any) => {
                element.forEach(async (element: any) => {
                    const productImageUrl: any = await this.doSpaceService.uploadFile(element, 'Product/');
                    const productImage = {
                        productId: product.id,
                        productName: product.name,
                        image: productImageUrl,
                    }
                    const productImgGallery = await this.productImageGalleryRepo.createEntity(productImage as CreateProductImageGalleryDto)
                });
            });
        }
        // end of product image processiong 

        return ResponseUtils.successResponseHandler(200, "Product created successfully!", "data", product);
    }



    async findAll(take: number, page: number): Promise<Product[] | Error> {
        const data = await this.repository.findAllEntities(take, page);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product fetched successfully", "data", data);
    }

    async findNewArrival(count: number): Promise<Product[] | Error> {
        const data = await this.repository.findEntitiesByLimit(count);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Successfully fetched", "products", data);
    }


    async populateProducts(productCategories: any, data: any) {
        try {
            await Promise.all(productCategories.map(async (element: any) => {
                const products: any = await this.repository.findProductByParentCategoryRepo(element.id, 5, 1);
                if (products.data.length > 0) {
                    data[`${element.name}`] = products;
                }
            }));

            return data;
        } catch (e) {
            throw e;
        }
    }


    async findHomePageDetails(count: number): Promise<any | Error> {
        const newArrival = await this.repository.findEntitiesByLimit(count);
        const findFeaturedProduct = await this.repository.findFeaturedProduct(count);
        const findAllProducts = await this.repository.findAllEntities(5, 1);

        if (!newArrival || !findFeaturedProduct || !findAllProducts) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = { newArrival, findFeaturedProduct, findAllProducts }
        return data;
    }

    async findFeaturedProduct(count: number): Promise<Product[] | Error> {
        const data = await this.repository.findFeaturedProduct(count);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Successfully fetched", "products", data);
    }


    async findProductByBrand(brandName: string, limit: number, page: number, categoryId: String,
        maxPrice: number,
        minPrice: number,
        thickness: any, sortBy: any): Promise<any | Error> {
        const data = await this.repository.findProductByBrand(brandName, limit, page, categoryId, maxPrice, minPrice, thickness, sortBy);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Successfully fetched", "products", data);
    }

    async findProductByMenu(routeName: string, limit: number, page: number,
        categoryId: String,
        brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        console.log('Service = ', routeName);
        const data = await this.repository.findProductByMenu(routeName, limit, page, categoryId, brand, maxPrice, minPrice, thickness);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product data successfully fetched.", "products", data);
    }


    async findOne(id: string): Promise<Product | Error> {
        const data = await this.repository.findOneEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product data fetched Successfully!", "data", data);
    }


    async updateOneProduct(
        id: string,
        dto: UpdateProductDto,
        files: {
            productImages?: UploadedMulterFileI,
            featuredImage?: UploadedMulterFileI,
        }
    ): Promise<Product | Error> {
        if (files && files.featuredImage) {
            const featuredImage: any = await this.doSpaceService.uploadFile(files.featuredImage[0], 'Product/');
            dto.featuredImage = featuredImage;
        }

        const productObj = await this.repository.findOneRawEntity(id);
        if (!productObj) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        
        const productEntity = {
            name: dto.name,
            featuredImage: dto.featuredImage === null ? productObj.featuredImage : dto.featuredImage,
            sku: dto.sku,
            rating: dto.rating,
            price: dto.price,
            cost: dto.cost,
            stock: parseInt(dto.stock),
            weight: dto.weight ? dto.weight : 0,
            vendorName: dto.vendorName,
            parentCategoryId: dto.parentCategoryId,
            parentCategoryName: dto.parentCategoryName,
            subCategoryId: dto.subCategoryId,
            subCategoryName: dto.subCategoryName,
            childCategoryId: dto.childCategoryId,
            childCategoryName: dto.childCategoryName,
            isFeatured: dto.isFeatured === null ? productObj.isFeatured : dto.isFeatured,
            isDeactive: dto.isDeactive === null ? productObj.isDeactive : dto.isDeactive,
            shortDescription: dto.shortDescription === null ? productObj.shortDescription : dto.shortDescription,
            longDescription: dto.longDescription === null ? productObj.longDescription : dto.longDescription,
            isMerch: dto.isMerch === null ? productObj.isMerch : dto.isMerch,
            isGiftGuide: dto.isGiftGuide === null ? productObj.isGiftGuide : dto.isGiftGuide,
            isReadyToShip: dto.isReadyToShip === null ? productObj.isReadyToShip : dto.isReadyToShip
        }
        const updateProduct = await this.repository.updateEntity(id, productEntity);
        console.log("updateProduct=", updateProduct)

        let currentSizes: any = [];
        let currentColor: any = [];
        let currentTag: any = [];
        let currentHighlight: any = [];
        let currentMaterial: any = [];
        const data = productObj;

        // working with product size
        if (data.productSizeVariants && data.productSizeVariants.length > 0) {
            data.productSizeVariants.forEach(async (element: any) => {
                currentSizes.push(element.sizeVariants);
            });
        }

        if (dto.size && dto.size.length > 0) {
            const arrayCheck = await isArraySame(currentSizes, dto.size);
            if (!arrayCheck) {
                let newSizeEntity: any = [];
                dto.size.forEach(async (element: any) => {
                    const sizeData = await this.productSizeRepo.findByName(element);
                    newSizeEntity.push({
                        productId: id,
                        productName: productObj.name,
                        sizeId: sizeData[0].id,
                        sizeName: element
                    })
                });
                await this.productSizeVariantsRepo.updateProductSizeValues(id, newSizeEntity)
            }
        }
        // end of product size process 

        // processing color 
        if (data.productColorVariants && data.productColorVariants.length > 0) {
            data.productColorVariants.forEach(async (element: any) => {
                currentColor.push(element.colorVariants);
            });
        }

        if (dto.color && dto.color.length > 0) {
            const arrayCheck = await isArraySame(currentColor, dto.color);
            if (!arrayCheck) {
                let newColorEntity: any = [];
                dto.color.forEach(async (element: any) => {
                    const colorData = await this.productColorRepo.findByName(element);
                    newColorEntity.push({
                        productId: id,
                        productName: productObj.name,
                        colorId: colorData[0].id,
                        colorName: element,
                        colorCode: colorData[0].colorCode
                    })
                });
                await this.productColorVariantsRepo.updateProductColorValues(id, newColorEntity)
            }
        }
        // end of processing color 

        // processing tag 
        if (data.productTagVariants && data.productTagVariants.length > 0) {
            data.productTagVariants.forEach(async (element: any) => {
                currentTag.push(element.tagVariants);
            });
        }

        if (dto.tag && dto.tag.length > 0) {
            const arrayCheck = await isArraySame(currentTag, dto.tag);
            if (!arrayCheck) {
                let newTagEntity: any = [];
                dto.tag.forEach(async (element: any) => {
                    const tagData = await this.productTagRepo.findByName(element);
                    newTagEntity.push({
                        productId: id,
                        productName: productObj.name,
                        tagId: tagData[0]['id'],
                        tagName: element
                    })
                });
                await this.productTagVariantsRepo.updateProductTagValues(id, newTagEntity)
            }
        }
        // end of processing tag

        // processing highlight 
        if (data.productHighlightVariants && data.productHighlightVariants.length > 0) {
            data.productHighlightVariants.forEach(async (element: any) => {
                currentHighlight.push(element.highlightVariants);
            });
        }

        if (dto.highlight && dto.highlight.length > 0) {
            const arrayCheck = await isArraySame(currentHighlight, dto.highlight);
            if (!arrayCheck) {
                let newHighlightEntity: any = [];
                dto.highlight.forEach(async (element: any) => {
                    const highlightData = await this.productHighlightRepo.findByName(element);
                    newHighlightEntity.push({
                        productId: id,
                        productName: productObj.name,
                        highlightId: highlightData[0]['id'],
                        highlightName: element
                    })
                });
                await this.productHighlightVariantsRepo.updateProductHighlightValues(id, newHighlightEntity)
            }
        }
        // end of processing highlight

        // processing material 
        if (data.productMaterialVariants && data.productMaterialVariants.length > 0) {
            data.productMaterialVariants.forEach(async (element: any) => {
                currentMaterial.push(element.materialVariants);
            });
        }

        if (dto.material && dto.material.length > 0) {
            const arrayCheck = await isArraySame(currentMaterial, dto.material);
            if (!arrayCheck) {
                let newMaterialEntity: any = [];
                dto.material.forEach(async (element: any) => {
                    const materialData = await this.productMaterialRepo.findByName(element);
                    newMaterialEntity.push({
                        productId: id,
                        productName: productObj.name,
                        materialId: materialData[0]['id'],
                        materialName: element
                    })
                });
                await this.productMaterialVariantsRepo.updateProductMaterialValues(id, newMaterialEntity)
            }
        }
        // end of processing material

        if (files && files.productImages) {
            const productFile = files['productImages'];

            Array(productFile).forEach(async (element: any) => {
                element.forEach(async (element: any) => {
                    let productNewImage: any = await this.doSpaceService.uploadFile(element, 'Product/');
                    const productImage = {
                        productId: id,
                        productName: productObj.name,
                        image: productNewImage,
                    }
                    await this.productImageGalleryRepo.createEntity(productImage);
                });
            });
        }

        if (dto.deletedImage && dto.deletedImage.length > 0) {
            await this.productImageGalleryRepo.deleteImage(dto.deletedImage);
        }

        return ResponseUtils.successResponseHandler(200, "Product updated successfully!", "product", data);
    }


    async filterByProductName(
        limit: any,
        page: any,
        keyword: any,
        routeName: any,
        parentCategoryId: any,
        subCategoryId: any,
        childCategoryId: any,
        maxPrice: any,
        minPrice: any,
        color: any,
        size: any,
        tag: any,
        material: any,
        highlight: any,
        sortBy: any): Promise<any | Error> {
        const data = await this.repository.filterByProductsName(
            limit,
            page,
            keyword,
            routeName,
            parentCategoryId,
            subCategoryId,
            childCategoryId,
            maxPrice,
            minPrice,
            color,
            size,
            tag,
            material,
            highlight,
            sortBy
        );

        //console.log(data);

        

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        if(data.data.length > 0)
        {
            for (let index = 0; index < data.data.length; index++) {
                const element = data.data[index];
                console.log('element '+index+'= ', element);
                let itemDetail = await this.repository.findOne({
                    where: { id: element.id, isDeleted: false },
                    relations: [
                        'productColorVariants',
                        'productSizeVariants',
                        'productTagVariants',
                        'productMaterialVariants',
                        'productHighlightVariants'
                    ]
                });
                data.data[index]['itemDetail'] = itemDetail;
            }
        }

        return ResponseUtils.successResponseHandler(200, "Product Successfully fetched", "data", data);
    }


    async productFilterByParams(
        limit: number,
        page: number,
        categoryId: String,
        brandName: any,
        maxPrice: number,
        minPrice: number,
    ): Promise<Product[] | Error> {
        const data = await this.repository.productsFilterByParams(
            limit,
            page,
            categoryId,
            brandName,
            maxPrice,
            minPrice
        );
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product data successfully fetched", "products", data);
    }

    async remove(id: string): Promise<any | Error> {
        const data = await this.repository.deleteEntity(id);
        const photoDelete = await this.productImageGalleryRepo.deleteImageByProductId(id);

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(
            data,
            200,
            "Product data deleted successfully!"
        );
    }


    private cleanNameString(stringName: string) {
        return stringName.trim().replace(/[^a-zA-Z0-9 ]/g, '');
    }

    async productReOrderService(
        limit: number,
        page: number,
        categoryId: String,
    ): Promise<any | Error> {
        const data = await this.repository.productReorderRepository(limit, page, categoryId);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Product Re-Order data successfully fetched", "products", data);
    }

    async importProductStockService(dto: CreateImportProductDto, file: Express.Multer.File): Promise<any> {
        let fileData = file;
        const stocks = await this.productStockImportRepo.findStockIdEntity();
        const StockID = stocks ? await generateStockId(stocks) : 'goldengal';
        
        
        const excelDataRow = await readXlsxFile(file.path).then((rows) => {
            let excelDataRowDump = [];


            rows.forEach(async (element, index) => {
                console.log('element = ', element)
                if (index > 0) {
                    let prodId: string = element[0].toString();

                    let productRawData = await {
                        'StockID': StockID,
                        'Pid': prodId,
                        'Name': element[1],
                        'Sku': element[2],
                        'Price': element[3],
                        'CurrentStock': 0,
                        'NewStock': element[4],
                        'NewTotalStock': 0,
                        'QuantityReorder': element[5],
                        'Weight': element[6],
                        'Size': element[7],
                        'SizeIDs': [],
                        'Color': element[8],
                        'ColorIDs': [],
                        'Tag': element[9],
                        'TagIDs': [],
                        'Material': element[10],
                        'MaterialIDs': [],
                        'Highlight': element[11],
                        'HighlightIDs': [],
                        'ParentCategoryName': element[12],
                        'SubCategoryName': element[13],
                        'ChildCategoryName': element[14],
                        'Status': element[15]
                    };
                    //productRawData.ColorIDs = colorsArrayId;
                    excelDataRowDump.push(productRawData);


                }
            });
            return excelDataRowDump;
        })

        for (let index = 0; index < excelDataRow.length; index++) {
            const rowsData = excelDataRow[index];

            // Colors Section Start
            let colorsArray = [];
            if (JSON.stringify(rowsData.Color).length > 0) {
                let colorsData = JSON.stringify(rowsData.Color).trim();
                colorsArray = colorsData.split(',')
                console.log('colorsArray = ', colorsArray)


            }

            for (let j = 0; j < colorsArray.length; j++) {
                const colorsName = colorsArray[j];
                console.log('colorsName = ', colorsName);

                let colorsRawData = await this.productColorRepo.findByFilterQuery({
                    colorName: this.cleanNameString(colorsName).trim()
                });

                if (!colorsRawData) {
                    let clrDto = {
                        name: this.cleanNameString(colorsName).trim(),
                        colorCode: '#000000'
                    };
                    let newColorData = await this.productColorRepo.createEntity(clrDto as CreateProductColorDto)
                    excelDataRow[index].ColorIDs.push(newColorData);
                } else {
                    if (Object.keys(colorsRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(colorsName).trim(),
                            colorCode: '#000000'
                        };
                        let newColorData = await this.productColorRepo.createEntity(clrDto as CreateProductColorDto)
                        excelDataRow[index].ColorIDs.push(newColorData);
                    } else {
                        excelDataRow[index].ColorIDs.push(colorsRawData[0]);
                    }

                }
            }
            //Colors Section End



            //Size Section Start
            let sizeArray = [];
            if (JSON.stringify(rowsData.Size).length > 0) {
                let sizeData = JSON.stringify(rowsData.Size).trim();
                sizeArray = sizeData.split(',');
                console.log('sizeArray = ', sizeArray);
            }

            for (let j = 0; j < sizeArray.length; j++) {
                const sizeName = sizeArray[j];
                console.log('sizeName = ', sizeName);

                let sizeRawData = await this.productSizeRepo.findByFilterQuery({
                    name: this.cleanNameString(sizeName.trim())
                });

                if (!sizeRawData) {
                    let clrDto = {
                        name: this.cleanNameString(sizeName.trim()),
                    };
                    let newSizeData = await this.productSizeRepo.createEntity(clrDto as CreateProductSizeDto)
                    excelDataRow[index].SizeIDs.push(newSizeData);
                } else {
                    if (Object.keys(sizeRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(sizeName.trim())
                        };
                        let newSizeData = await this.productSizeRepo.createEntity(clrDto as CreateProductSizeDto)
                        excelDataRow[index].SizeIDs.push(newSizeData);
                    } else {
                        excelDataRow[index].SizeIDs.push(sizeRawData[0]);
                    }

                }

            }
            //Size Section End


            //Tag Section Start
            let tagArray = [];
            if (JSON.stringify(rowsData.Tag).length > 0) {
                let tagData = JSON.stringify(rowsData.Tag).trim();
                tagArray = tagData.split(',');
                console.log('tagArray = ', tagArray);
            }

            for (let j = 0; j < tagArray.length; j++) {
                const tagName = tagArray[j];
                console.log('tagName = ', tagName);

                let tagRawData = await this.productTagRepo.findByFilterQuery({
                    name: this.cleanNameString(tagName.trim())
                });

                if (!tagRawData) {
                    let clrDto = {
                        name: this.cleanNameString(tagName.trim()),
                    };
                    let newTagData = await this.productTagRepo.createEntity(clrDto as CreateProductTagDto)
                    excelDataRow[index].TagIDs.push(newTagData);
                } else {
                    if (Object.keys(tagRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(tagName.trim())
                        };
                        let newTagData = await this.productTagRepo.createEntity(clrDto as CreateProductTagDto)
                        excelDataRow[index].TagIDs.push(newTagData);
                    } else {
                        excelDataRow[index].TagIDs.push(tagRawData[0]);
                    }

                }
            }
            //Tag Section End

            //Material Section Start
            let materialArray = [];
            if (JSON.stringify(rowsData.Material).length > 0) {
                let materialData = JSON.stringify(rowsData.Material).trim();
                materialArray = materialData.split(',');
                console.log('materialArray = ', materialArray);
            }

            for (let j = 0; j < materialArray.length; j++) {
                const materialName = materialArray[j];
                console.log('materialName = ', materialName);

                let materialRawData = await this.productMaterialRepo.findByFilterQuery({
                    name: this.cleanNameString(materialName.trim())
                });

                if (!materialRawData) {
                    let clrDto = {
                        name: this.cleanNameString(materialName.trim()),
                    };
                    let newMaterialData = await this.productMaterialRepo.createEntity(clrDto as CreateProductMaterialDto)
                    excelDataRow[index].MaterialIDs.push(newMaterialData);
                } else {
                    if (Object.keys(materialRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(materialName.trim())
                        };
                        let newMaterialData = await this.productMaterialRepo.createEntity(clrDto as CreateProductMaterialDto)
                        excelDataRow[index].MaterialIDs.push(newMaterialData);
                    } else {
                        excelDataRow[index].MaterialIDs.push(materialRawData[0]);
                    }
                }
            }
            //Material Section End


            //Highlight Section Start
            let highlightArray = [];
            if (JSON.stringify(rowsData.Highlight).length > 0) {
                let highlightData = JSON.stringify(rowsData.Highlight).trim();
                highlightArray = highlightData.split(',')
                console.log('highlightArray = ', highlightArray)
            }

            for (let j = 0; j < highlightArray.length; j++) {
                const highlightName = highlightArray[j];
                console.log('highlightName = ', highlightName)

                let highlightRawData = await this.productHighlightRepo.findByFilterQuery({
                    name: this.cleanNameString(highlightName.trim())
                });

                if (!highlightRawData) {
                    let clrDto = {
                        name: this.cleanNameString(highlightName.trim()),
                    };
                    let newHighlightData = await this.productHighlightRepo.createEntity(clrDto as CreateProductHighlightDto)
                    excelDataRow[index].HighlightIDs.push(newHighlightData);
                } else {
                    if (Object.keys(highlightRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(highlightName.trim())
                        };
                        let newHighlightData = await this.productHighlightRepo.createEntity(clrDto as CreateProductHighlightDto)
                        excelDataRow[index].HighlightIDs.push(newHighlightData);
                    } else {
                        excelDataRow[index].HighlightIDs.push(highlightRawData[0]);
                    }
                }
            }
            //Material Section End
        }
        
        console.log("excelDataRow = ", excelDataRow);
        
        for (let k = 0; k < excelDataRow.length; k++) {
            const element = excelDataRow[k];
            const id = element.Pid;
            let productInfo = await this.repository.findOne(id);
            if (typeof productInfo === 'undefined') {
                console.log('Index product Not found = ' + k, element.Pid);
            } else {
                console.log('Index product found = ' + k, productInfo);
                let currentQuantity = productInfo["stock"];
                let newQuantity = element.NewStock;
                let totalQuantity = currentQuantity + newQuantity;

                excelDataRow[k]['CurrentStock'] = currentQuantity;
                excelDataRow[k]['NewTotalStock'] = totalQuantity;
            }
        }

        excelDataRow.forEach(async row => {
            console.log("row=>", row);

            let dataRow = {
                'StockID': row.StockID,
                'Pid': row.Pid,
                'Name': row.Name,
                'Price': row.Price,
                'CurrentStock': row.CurrentStock,
                'NewStock': row.NewStock,
                'NewTotalStock': row.NewTotalStock,
                'QuantityReorder': row.QuantityReorder,
                'Sku': row.Sku,
                'Weight': row.Weight,
                'Size': row.Size,
                'Color': row.Color,
                'Tag': row.Tag,
                'Material': row.Material,
                'Highlight': row.Highlight,
                'ParentCategory': row.ParentCategoryName,
                'SubCategory': row.SubCategoryName,
                'ChildCategory': row.ChildCategoryName,
                'Status': row.Status
                
            };
            const data = await this.productStockImportRepo.createEntity(dataRow as CreateProductStockImportDto);
            console.log("data=>", data);

        });

        let dataRetrn = {
            stockID: StockID,
            stock: excelDataRow
        }

        return ResponseUtils.successResponseHandler(200, "Import Product Data Stock Successfully created", "product_stock", dataRetrn);
    }


    async confirmStockInProductService(dto: CreateConfirmStockInProductDto): Promise<any> {
        if (dto.stockID.length > 0) {
            let stockData = await this.productStockImportRepo.findByFilterQuery({
                StockID: dto.stockID
            });
            console.log("stockData = ", stockData);
            if (!stockData) {
                return ResponseUtils.errorResponseHandler(404, "Stock information not available");
            }

            for (let index = 0; index < Object.keys(stockData).length; index++) {
                const row = stockData[index];
                console.log('Stock ROw = ', row);

                let crProd = {
                    'name': row.Name,
                    'price': row.Price,
                    'stock': row.NewTotalStock,
                    'quantityReorder': row.QuantityReorder,
                    'weight': row.Weight,
                    'sku': row.Sku,
                    'isFeatured': row.isFeatured,
                    'featuredImage': row.featuredImage,
                    'size': row.Size,
                    'color': row.Color,
                    'tag': row.Tag,
                    'material': row.Material,
                    'highlight': row.Highlight,
                    'parentCategoryName': row.ParentCategoryName,
                    'parentCategoryId': row.ParentCategoryName,
                    'subCategoryName': row.SecondCategory,
                    'subCategoryId': row.SecondCategory,
                    'childCategoryName': row.ChildCategoryName,
                    'childCategoryId': row.ChildCategoryName,
                    'deletedImage': null
                }

                let parentCategoryData = await this.productParentCatRepo.findAllEntitiesSelect({
                    name: row.ParentCategory
                })
                console.log("+++parentCategoryData+++: ",parentCategoryData);
                crProd.parentCategoryName = !parentCategoryData || Object.keys(parentCategoryData).length == 0 ? 0 : parentCategoryData[0].name;
                crProd.parentCategoryId = !parentCategoryData || Object.keys(parentCategoryData).length == 0 ? 0 : parentCategoryData[0].id;

                let subCategoryData = await this.productSubCatRepo.findAllEntitiesSelect({
                    name: row.SubCategory
                })
                console.log("+++subCategoryData+++: ",subCategoryData);
                crProd.subCategoryName = !subCategoryData || Object.keys(subCategoryData).length == 0 ? 0 : subCategoryData[0].name;
                crProd.subCategoryId = !subCategoryData || Object.keys(subCategoryData).length == 0 ? 0 : subCategoryData[0].id;

                let childCategoryData = await this.productChildCatRepo.findAllEntitiesSelect({
                    name: row.ChildCategory
                })
                console.log("+++childCategoryData+++: ",childCategoryData);
                crProd.childCategoryName = !childCategoryData || Object.keys(childCategoryData).length == 0 ? 0 : childCategoryData[0].name;
                crProd.childCategoryId = !childCategoryData || Object.keys(childCategoryData).length == 0 ? 0 : childCategoryData[0].id;

                //Colors Section Start
                let colorsArray = [];
                if (JSON.stringify(row.Color).length > 0) {
                    let colorsData = JSON.stringify(row.Color).trim();
                    let colorsArrayRaw = colorsData.split(',');
                    console.log('colorsArray = ', colorsArrayRaw);

                    for (let index = 0; index < colorsArrayRaw.length; index++) {
                        const element = this.cleanNameString(colorsArrayRaw[index]);
                        colorsArray.push(element);
                    }
                    crProd.color = (colorsArray.length > 0) ? colorsArray : [];
                }
                //Colors Section End


                //Size Section Start
                let sizeArray = [];
                if (JSON.stringify(row.Size).length > 0) {
                    let sizeData = JSON.stringify(row.Size).trim();
                    let sizeArrayRaw = sizeData.split(',');
                    console.log('sizeArrayRaw = ', sizeArrayRaw);

                    for (let index = 0; index < sizeArrayRaw.length; index++) {
                        const element = this.cleanNameString(sizeArrayRaw[index]);
                        sizeArray.push(element);
                    }
                    crProd.size = (sizeArray.length > 0) ? sizeArray : [];
                }
                // Size Section End
                
                //Tag Section Start
                let tagArray = [];
                if (JSON.stringify(row.Tag).length > 0) {
                    let tagData = JSON.stringify(row.Tag).trim();
                    let tagArrayRaw = tagData.split(',');
                    console.log('tagArrayRaw = ', tagArrayRaw);
                    

                    for (let index = 0; index < tagArrayRaw.length; index++) {
                        const element = this.cleanNameString(tagArrayRaw[index]);
                        tagArray.push(element);
                    }
                    crProd.tag = (tagArray.length > 0) ? tagArray : [];
                }
                //Tag Section End

                //Material Section Start
                let materialArray = [];
                if (JSON.stringify(row.Material).length > 0) {
                    let materialData = JSON.stringify(row.Material).trim();
                    let materialArrayRaw = materialData.split(',')
                    console.log('materialArrayRaw = ', materialArrayRaw)

                    for (let index = 0; index < materialArrayRaw.length; index++) {
                        const element = this.cleanNameString(materialArrayRaw[index]);
                        materialArray.push(element);
                    }
                    crProd.material = (materialArray.length > 0) ? materialArray : [];
                }
                //Material Section End

                //Highlight Section Start
                let highlightArray = [];
                if (JSON.stringify(row.Highlight).length > 0) {
                    let highlightData = JSON.stringify(row.Highlight).trim();
                    let highlightArrayRaw = highlightData.split(',')
                    console.log('highlightArrayRaw = ', highlightArrayRaw)

                    for (let index = 0; index < highlightArrayRaw.length; index++) {
                        const element = this.cleanNameString(highlightArrayRaw[index]);
                        highlightArray.push(element);
                    }
                    crProd.highlight = (highlightArray.length > 0) ? highlightArray : [];
                }
                //Highlight Section End
                console.log('before crProd =', crProd)

                let proCreate = await this.updateOneProduct(row.Pid, crProd as UpdateProductDto, {})

                console.log('crProd =', crProd)
                console.log('proCreate =', proCreate)
            }

        }

        return ResponseUtils.successResponseHandler(200, "Confirm Stockin Product Data Successfully Completed", "product_stockin_product", null);
    }


    async importProductService(dto: CreateImportProductDto, file: Express.Multer.File): Promise<any> {
        let fileData = file;
        let dumpID = uuidv4();
        const excelDataRow = await readXlsxFile(file.path).then((rows) => {
            let excelDataRowDump = [];

            rows.forEach(async element => {

                if (element[0] != "Name") {
                    let productRawData = {
                        'DumpID': dumpID,
                        'Name': element[0],
                        'Sku': element[1],
                        'Price': element[2],
                        'Stock': element[3],
                        'QuantityReorder': element[4],
                        'Weight': element[5],
                        'Size': element[6],
                        'SizeIDs': [],
                        'Color': element[7],
                        'ColorIDs': [],
                        'Tag': element[8],
                        'TagIDs': [],
                        'Material': element[9],
                        'MaterialIDs': [],
                        'Highlight': element[10],
                        'HighlightIDs': [],
                        'ParentCategoryName': element[11],
                        'SubCategoryName': element[12],
                        'ChildCategoryName': element[13],
                        'Status': element[14]
                    };
                    excelDataRowDump.push(productRawData);
                }
            });

            return excelDataRowDump;
        })

        for (let index = 0; index < excelDataRow.length; index++) {
            const rowsData = excelDataRow[index];

            //Colors Section Start
            let colorsArray = [];
            if (JSON.stringify(rowsData.Color).length > 0) {
                let colorsData = JSON.stringify(rowsData.Color).trim();
                colorsArray = colorsData.split(',')
                console.log('colorsArray = ', colorsArray)


            }

            for (let j = 0; j < colorsArray.length; j++) {
                const colorsName = colorsArray[j];
                console.log('colorsArray = ', colorsName)

                let colorsRawData = await this.productColorRepo.findByFilterQuery({
                    name: this.cleanNameString(colorsName).trim(),
                    colorCode: '#0000'
                });

                if (!colorsRawData) {
                    let clrDto = {
                        name: this.cleanNameString(colorsName).trim(),
                        colorCode: '#0000'
                    };
                    let newColorData = await this.productColorRepo.createEntity(clrDto as CreateProductColorDto)
                    excelDataRow[index].ColorIDs.push(newColorData);
                } else {
                    if (Object.keys(colorsRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(colorsName).trim(),
                            colorCode: '#0000'
                        };
                        let newColorData = await this.productColorRepo.createEntity(clrDto as CreateProductColorDto)
                        excelDataRow[index].ColorIDs.push(newColorData);
                    } else {
                        excelDataRow[index].ColorIDs.push(colorsRawData[0]);
                    }

                }

            }
            //Colors Section End



            // //Size Section Start
            let sizeArray = [];
            if (JSON.stringify(rowsData.Size).length > 0) {
                let sizeData = JSON.stringify(rowsData.Size).trim();
                sizeArray = sizeData.split(',')
                console.log('sizeArray = ', sizeArray)
            }

            for (let j = 0; j < sizeArray.length; j++) {
                const sizeName = sizeArray[j];
                console.log('sizeName = ', sizeName)

                let sizeRawData = await this.productSizeRepo.findByFilterQuery({
                    name: this.cleanNameString(sizeName.trim())
                });

                if (!sizeRawData) {
                    let sizeDto = {
                        name: this.cleanNameString(sizeName.trim()),

                    };
                    let newSizeData = await this.productSizeRepo.createEntity(sizeDto as CreateProductSizeDto)
                    excelDataRow[index].SizeIDs.push(newSizeData);
                } else {
                    if (Object.keys(sizeRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(sizeName.trim()),

                        };
                        let newSizeData = await this.productSizeRepo.createEntity(clrDto as CreateProductSizeDto)
                        excelDataRow[index].SizeIDs.push(newSizeData);
                    } else {
                        excelDataRow[index].SizeIDs.push(sizeRawData[0]);
                    }

                }

            }
            //Size Section End

            //Tag Section Start
            let tagArray = [];
            if (JSON.stringify(rowsData.Tag).length > 0) {
                let tagData = JSON.stringify(rowsData.Tag).trim();
                tagArray = tagData.split(',');
                console.log('tagArray = ', tagArray);
            }

            for (let j = 0; j < tagArray.length; j++) {
                const tagName = tagArray[j];
                console.log('tagName = ', tagName);

                let tagRawData = await this.productTagRepo.findByFilterQuery({
                    name: this.cleanNameString(tagName.trim())
                });

                if (!tagRawData) {
                    let clrDto = {
                        name: this.cleanNameString(tagName.trim()),
                    };
                    let newTagData = await this.productTagRepo.createEntity(clrDto as CreateProductTagDto)
                    excelDataRow[index].TagIDs.push(newTagData);
                } else {
                    if (Object.keys(tagRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(tagName.trim())
                        };
                        let newTagData = await this.productTagRepo.createEntity(clrDto as CreateProductTagDto)
                        excelDataRow[index].TagIDs.push(newTagData);
                    } else {
                        excelDataRow[index].TagIDs.push(tagRawData[0]);
                    }

                }
            }
            //Tag Section End

            //Material Section Start
            let materialArray = [];
            if (JSON.stringify(rowsData.Material).length > 0) {
                let materialData = JSON.stringify(rowsData.Material).trim();
                materialArray = materialData.split(',');
                console.log('materialArray = ', materialArray);
            }

            for (let j = 0; j < materialArray.length; j++) {
                const materialName = materialArray[j];
                console.log('materialName = ', materialName);

                let materialRawData = await this.productMaterialRepo.findByFilterQuery({
                    name: this.cleanNameString(materialName.trim())
                });

                if (!materialRawData) {
                    let clrDto = {
                        name: this.cleanNameString(materialName.trim()),
                    };
                    let newMaterialData = await this.productMaterialRepo.createEntity(clrDto as CreateProductMaterialDto)
                    excelDataRow[index].MaterialIDs.push(newMaterialData);
                } else {
                    if (Object.keys(materialRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(materialName.trim())
                        };
                        let newMaterialData = await this.productMaterialRepo.createEntity(clrDto as CreateProductMaterialDto)
                        excelDataRow[index].MaterialIDs.push(newMaterialData);
                    } else {
                        excelDataRow[index].MaterialIDs.push(materialRawData[0]);
                    }
                }
            }
            //Material Section End


            //Highlight Section Start
            let highlightArray = [];
            if (JSON.stringify(rowsData.Highlight).length > 0) {
                let highlightData = JSON.stringify(rowsData.Highlight).trim();
                highlightArray = highlightData.split(',')
                console.log('highlightArray = ', highlightArray)
            }

            for (let j = 0; j < highlightArray.length; j++) {
                const highlightName = highlightArray[j];
                console.log('highlightName = ', highlightName)

                let highlightRawData = await this.productHighlightRepo.findByFilterQuery({
                    name: this.cleanNameString(highlightName.trim())
                });

                if (!highlightRawData) {
                    let clrDto = {
                        name: this.cleanNameString(highlightName.trim()),
                    };
                    let newHighlightData = await this.productHighlightRepo.createEntity(clrDto as CreateProductHighlightDto)
                    excelDataRow[index].HighlightIDs.push(newHighlightData);
                } else {
                    if (Object.keys(highlightRawData).length == 0) {
                        let clrDto = {
                            name: this.cleanNameString(highlightName.trim())
                        };
                        let newHighlightData = await this.productHighlightRepo.createEntity(clrDto as CreateProductHighlightDto)
                        excelDataRow[index].HighlightIDs.push(newHighlightData);
                    } else {
                        excelDataRow[index].HighlightIDs.push(highlightRawData[0]);
                    }
                }
            }
            //Material Section End
        }

        excelDataRow.forEach(async row => {
            console.log(row);
            let dataRow = {
                'DumpID': row.DumpID,
                'Name': row.Name,
                'Sku': row.Sku,
                'Price': row.Price,
                'Stock': row.Stock,
                'QuantityReorder': row.QuantityReorder,
                'Weight': row.Weight,
                'Size': row.Size,
                'Color': row.Color,
                'Tag': row.Tag,
                'Material': row.Material,
                'Highlight': row.Highlight,
                'ParentCategory': row.ParentCategoryName,
                'SubCategory': row.SubCategoryName,
                'ChildCategory': row.ChildCategoryName,
                'Status': row.Status
            };
            await this.productDumpImportRepo.createEntity(dataRow as CreateProductDumpImportDto);

        });


        let dataRetrn = {
            dumpID: dumpID,
            dump: excelDataRow
        }


        return ResponseUtils.successResponseHandler(200, "Import Product Data Dump Successfully created", "product_dump", dataRetrn);
    }

    async confirmImportProductService(dto: CreateConfirmImportProductDto): Promise<any> {
        if (dto.dumpID.length > 0) {
            let dumpData = await this.productDumpImportRepo.findByFilterQuery({
                DumpID: dto.dumpID
            });
            console.log(dumpData);

            if (!dumpData) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }

            for (let index = 0; index < Object.keys(dumpData).length; index++) {
                const row = dumpData[index];
                console.log('ROw = ', row);



                let crProd = {
                    'name': row.Name,
                    'sku': row.Sku,
                    // 'cost': row.Cost,
                    'price': row.Price,
                    'stock': row.Stock,
                    'weight': row.Weight,
                    'size': row.Size,
                    'color': row.Color,
                    'tag': row.Tag,
                    'material': row.Material,
                    'highlight': row.Highlight,
                    // 'brandName': row.BrandName,
                    'parentCategoryId': row.ParentCategory,
                    'subCategoryId': row.SubCategory,
                    'childCategoryId': row.ChildCategory,
                    'status': row.Status
                }


                let parentCategoryData = await this.productParentCatRepo.findAllEntitiesSelect({
                    name: row.ParentCategory
                })
                console.log("parentCategoryData=", parentCategoryData);
                crProd.parentCategoryId = !parentCategoryData || Object.keys(parentCategoryData).length == 0 ? 0 : parentCategoryData[0].id;
                crProd["parentCategoryName"] = row.ParentCategory;

                let subCategoryData = await this.productSubCatRepo.findAllEntitiesSelect({
                    name: row.SubCategory
                })
                console.log("subCategoryData=", subCategoryData);
                crProd.subCategoryId = !subCategoryData || Object.keys(subCategoryData).length == 0 ? 0 : subCategoryData[0].id;
                crProd["subCategoryName"] = row.SubCategory;

                let childCategoryData = await this.productChildCatRepo.findAllEntitiesSelect({
                    name: row.ChildCategory
                })
                console.log("childCategoryData=", childCategoryData);
                crProd.childCategoryId = !childCategoryData || Object.keys(childCategoryData).length == 0 ? 0 : childCategoryData[0].id;
                crProd["childCategoryName"] = row.ChildCategory;

                //Pulling color size temperature wattage weight start 
                //Colors Section Start
                let colorsArray = [];
                if (JSON.stringify(row.Color).length > 0) {
                    let colorsData = JSON.stringify(row.Color).trim();
                    let colorsArrayRaw = colorsData.split(',')
                    console.log('colorsArray = ', colorsArrayRaw)

                    for (let index = 0; index < colorsArrayRaw.length; index++) {
                        const element = this.cleanNameString(colorsArrayRaw[index]);
                        colorsArray.push(element);
                    }
                    crProd.color = (colorsArray.length > 0) ? colorsArray : [];
                }
                //Colors Section End


                //Size Section Start
                let sizeArray = [];
                if (JSON.stringify(row.Size).length > 0) {
                    let sizeData = JSON.stringify(row.Size).trim();
                    let sizeArrayRaw = sizeData.split(',')
                    console.log('sizeArrayRaw = ', sizeArrayRaw)

                    for (let index = 0; index < sizeArrayRaw.length; index++) {
                        const element = this.cleanNameString(sizeArrayRaw[index]);
                        sizeArray.push(element);
                    }
                    crProd.size = (sizeArray.length > 0) ? sizeArray : [];
                }
                //Size Section End

                //Tag Section Start
                let tagArray = [];
                if (JSON.stringify(row.Tag).length > 0) {
                    let tagData = JSON.stringify(row.Tag).trim();
                    let tagArrayRaw = tagData.split(',');
                    console.log('tagArrayRaw = ', tagArrayRaw);
                    

                    for (let index = 0; index < tagArrayRaw.length; index++) {
                        const element = this.cleanNameString(tagArrayRaw[index]);
                        tagArray.push(element);
                    }
                    crProd.tag = (tagArray.length > 0) ? tagArray : [];
                }
                //Tag Section End

                //Material Section Start
                let materialArray = [];
                if (JSON.stringify(row.Material).length > 0) {
                    let materialData = JSON.stringify(row.Material).trim();
                    let materialArrayRaw = materialData.split(',')
                    console.log('materialArrayRaw = ', materialArrayRaw)

                    for (let index = 0; index < materialArrayRaw.length; index++) {
                        const element = this.cleanNameString(materialArrayRaw[index]);
                        materialArray.push(element);
                    }
                    crProd.material = (materialArray.length > 0) ? materialArray : [];
                }
                //Material Section End

                //Highlight Section Start
                let highlightArray = [];
                if (JSON.stringify(row.Highlight).length > 0) {
                    let highlightData = JSON.stringify(row.Highlight).trim();
                    let highlightArrayRaw = highlightData.split(',')
                    console.log('highlightArrayRaw = ', highlightArrayRaw)

                    for (let index = 0; index < highlightArrayRaw.length; index++) {
                        const element = this.cleanNameString(highlightArrayRaw[index]);
                        highlightArray.push(element);
                    }
                    crProd.highlight = (highlightArray.length > 0) ? highlightArray : [];
                }
                //Highlight Section End

                let proCreate = await this.create(crProd as unknown as CreateProductDto, {})

                console.log('crProd =', crProd)
                console.log('proCreate =', proCreate)

            }

        }

        return ResponseUtils.successResponseHandler(200, "Confirm Import Product Data Dump Successfully Completed", "product_dump_create_product", null);
    }

    async exportProductService(): Promise<any | Error> {
        const data = await this.repository.exportAllProductRepo();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        for (let index = 0; index < Object.keys(data).length; index++) {
            const row = data[index];

            console.log('Color [' + index + ']= ', row.colors);
            let color_string = "";
            for (let j = 0; j < Object.keys(row.colors).length; j++) {
                const element = row.colors[j];
                if (color_string.length > 0) {
                    color_string += ",";
                }
                color_string += element.color;
            }
            data[index].colors = color_string;

            console.log('sizes [' + index + ']= ', row.sizes);
            let sizes_string = "";
            for (let j = 0; j < Object.keys(row.sizes).length; j++) {
                const element = row.sizes[j];
                if (sizes_string.length > 0) {
                    sizes_string += ",";
                }
                sizes_string += element.size;
            }
            data[index].sizes = sizes_string;

            let parentCat = row.parentCategoryId;
            const parentCatData = await this.productParentCatRepo.findOneEntity(parentCat);
            if (!parentCatData) {
                row['parentCategoryName'] = "";
            } else {
                row['parentCategoryName'] = parentCatData.name;
            }

            let subCat = row.subCategoryId;
            const subCatData = await this.productSubCatRepo.findOneEntity(subCat);
            if (!subCatData) {
                row['subCategoryName'] = "";
            } else {
                row['subCategoryName'] = subCatData.name;
            }

            let childCat = row.childCategoryId;
            const childCatData = await this.productChildCatRepo.findOneEntity(childCat);
            if (!childCatData) {
                row['childCategoryName'] = "";
            } else {
                row['childCategoryName'] = childCatData.name;
            }
        }

        return ResponseUtils.successResponseHandler(200, "Product Successfully fetched", "products", data);
    }

}
